@@ -494,7 +494,7 @@ class DFTNormalisation(DataFrameTransformer):
 
             if len(matchingColumns) > 0:
                 if rule.unsupported:
-                    raise Exception(f"Normalisation of columns {matchingColumns} is unsupported according to {rule}")
+                    raise Exception(f"Normalisation of columns {matchingColumns} is unsupported according to {rule}. If you want to make use of these columns, transform them into a supported column before applying {self.__class__.__name__}.")
                 if not rule.skip:
                     if rule.transformer is None:
                         if self._defaultTransformerFactory is None:
@@ -524,7 +524,7 @@ class DFTNormalisation(DataFrameTransformer):
         if self.requireAllHandled:
             unhandledColumns = set(df.columns) - set(matchedRulesByColumn.keys())
             if len(unhandledColumns) > 0:
-                raise Exception(f"The following columns are not handled by any rules: {unhandledColumns}")
+                raise Exception(f"The following columns are not handled by any rules: {unhandledColumns}; rules: {', '.join(map(str, self._rules))}")
 
     def _apply(self, df: pd.DataFrame) -> pd.DataFrame:
         if not self.inplace:
