@@ -17,13 +17,13 @@ import seaborn as sns
 
 from .crossval import PredictorModelCrossValidationData, VectorRegressionModelCrossValidationData, \
     VectorClassificationModelCrossValidationData, \
-    VectorClassificationModelCrossValidator, VectorRegressionModelCrossValidator
+    VectorClassificationModelCrossValidator, VectorRegressionModelCrossValidator, VectorModelCrossValidator
 from .eval_stats.eval_stats_base import EvalStats, EvalStatsCollection
 from .eval_stats.eval_stats_classification import ClassificationEvalStats
 from .eval_stats.eval_stats_regression import RegressionEvalStats
-from .evaluator import VectorModelEvaluator, PredictorModelEvaluationData, VectorRegressionModelEvaluator, \
+from .evaluator import PredictorModelEvaluator, PredictorModelEvaluationData, VectorRegressionModelEvaluator, \
     VectorRegressionModelEvaluationData, VectorClassificationModelEvaluator, VectorClassificationModelEvaluationData
-from ..data_ingest import InputOutputData
+from ..data import InputOutputData
 from ..util.io import ResultWriter
 from ..vector_model import VectorClassificationModel, VectorRegressionModel, VectorModel
 
@@ -32,7 +32,8 @@ log = logging.getLogger(__name__)
 TModel = TypeVar("TModel", bound=VectorModel)
 TEvalStats = TypeVar("TEvalStats", bound=EvalStats)
 TEvalStatsCollection = TypeVar("TEvalStatsCollection", bound=EvalStatsCollection)
-TEvaluator = TypeVar("TEvaluator", bound=VectorModelEvaluator)
+TEvaluator = TypeVar("TEvaluator", bound=PredictorModelEvaluator)
+TCrossValidator = TypeVar("TCrossValidator", bound=VectorModelCrossValidator)
 TEvalData = TypeVar("TEvalData", bound=PredictorModelEvaluationData)
 TCrossValData = TypeVar("TCrossValData", bound=PredictorModelCrossValidationData)
 
@@ -61,8 +62,8 @@ def createVectorModelCrossValidator(data: InputOutputData, model: VectorModel =
     return cons(data, folds=folds, **kwargs)
 
 
-def createEvaluationUtil(data: InputOutputData, model: VectorModel = None,
-        isRegression: bool = None, evaluatorParams: Optional[Dict[str, Any]] = None,
+def createEvaluationUtil(data: InputOutputData, model: VectorModel = None, isRegression: bool = None,
+        evaluatorParams: Optional[Dict[str, Any]] = None,
         crossValidatorParams: Optional[Dict[str, Any]] = None) \
             -> Union["ClassificationEvaluationUtil", "RegressionEvaluationUtil"]:
     cons = RegressionEvaluationUtil if _isRegression(model, isRegression) else ClassificationEvaluationUtil
@@ -110,7 +111,7 @@ def evalModelViaEvaluator(model: TModel, inputOutputData: InputOutputData, testF
     return ev.performSimpleEvaluation(model, showPlots=True, logResults=True)
 
 
-class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValData, TEvalStats]):
+class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValidator, TCrossValData, TEvalStats]):
     """
     Utility class for the evaluation of models based on a dataset
     """
@@ -144,25 +145,40 @@ class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValData,
                 resultWriter = resultWriter.childWithAddedPrefix(addedFilenamePrefix)
             return self.__class__(showPlots=self.showPlots, resultWriter=resultWriter)
 
-    def createEvaluator(self, model) -> TEvaluator:
+    def createEvaluator(self, model: TModel = None, isRegression: bool = None) -> TEvaluator:
         """
-        Creates an evaluator which is suitable for evaluation of the given model
-        :param model: the model for which to create an evaluator
+        Creates an evaluator holding the current input-output data
+
+        :param model: the model for which to create an evaluator (just for reading off regression or classification,
+            the resulting evaluator will work on other models as well)
+        :param isRegression: whether to create a regression model evaluator. Either this or model have to be specified
         :return: an evaluator
         """
-        return createVectorModelEvaluator(self.inputOutputData, model=model, **self.evaluatorParams)
+        return createVectorModelEvaluator(self.inputOutputData, model=model, isRegression=isRegression, **self.evaluatorParams)
+
+    def createCrossValidator(self, model: TModel = None, isRegression: bool = None) -> TCrossValidator:
+        """
+        Creates a cross-validator holding the current input-output data
+
+        :param model: the model for which to create a cross-validator (just for reading off regression or classification,
+            the resulting evaluator will work on other models as well)
+        :param isRegression: whether to create a regression model cross-validator. Either this or model have to be specified
+        :return: an evaluator
+        """
+        return createVectorModelCrossValidator(self.inputOutputData, model=model, isRegression=isRegression, **self.crossValidatorParams)
 
     def performSimpleEvaluation(self, model: TModel, showPlots=False, logResults=True, resultWriter: ResultWriter = None,
             additionalEvaluationOnTrainingData=False) -> TEvalData:
+        resultWriter = self._resultWriterForModel(resultWriter, model)
         evaluator = self.createEvaluator(model)
         evaluator.fitModel(model)
 
         def gatherResults(evalResultData, resultWriter, subtitlePrefix=""):
-            strEvalResults = ""
+            strEvalResults = f"{model}\n\n"
             for predictedVarName in model.getPredictedVariableNames():
                 strEvalResult = str(evalResultData.getEvalStats(predictedVarName))
                 if logResults:
-                    log.info(f"Evaluation results for {predictedVarName}: {strEvalResult}")
+                    log.info(f"{subtitlePrefix}Evaluation results for {predictedVarName}: {strEvalResult}")
                 strEvalResults += predictedVarName + ": " + strEvalResult + "\n"
             if resultWriter is not None:
                 resultWriter.writeTextFile("evaluator-results", strEvalResults)
@@ -172,7 +188,8 @@ class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValData,
         gatherResults(evalResultData, resultWriter)
         if additionalEvaluationOnTrainingData:
             evalResultDataTrain = evaluator.evalModel(model, onTrainingData=True)
-            gatherResults(evalResultDataTrain, resultWriter.childWithAddedPrefix("-onTrain-"), subtitlePrefix="[onTrain] ")
+            additionalResultWriter = resultWriter.childWithAddedPrefix("-onTrain-") if resultWriter is not None else None
+            gatherResults(evalResultDataTrain, additionalResultWriter, subtitlePrefix="[onTrain] ")
 
         return evalResultData
 
@@ -200,10 +217,37 @@ class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValData,
         if logResults:
             log.info(f"Cross-validation results: {strEvalResults}")
         if resultWriter is not None:
-            resultWriter.writeTextFile("evaluator-results", strEvalResults)
+            resultWriter.writeTextFile("crossval-results", strEvalResults)
         self.createPlots(crossValidationData, showPlots=showPlots, resultWriter=resultWriter)
         return crossValidationData
 
+    def compareModels(self, models: Sequence[TModel], resultWriter: Optional[ResultWriter] = None, useCrossValidation=False) -> pd.DataFrame:
+        """
+        Compares several models via simple evaluation or cross-validation
+
+        :param models: the models to compare
+        :param resultWriter: a writer with which to store results of the comparison
+        :return: a data frame containing evaluation metrics on all models
+        """
+        statsList = []
+        for model in models:
+            if useCrossValidation:
+                crossValidationResult = self.performCrossValidation(model, resultWriter=resultWriter)
+                statsDict = crossValidationResult.getEvalStatsCollection().aggStats()
+            else:
+                evalStats: EvalStats = self.performSimpleEvaluation(model, resultWriter=resultWriter).getEvalStats()
+                statsDict = evalStats.getAll()
+            statsDict["modelName"] = model.getName()
+            statsList.append(statsDict)
+        resultsDF = pd.DataFrame(statsList).set_index("modelName")
+        strResults = f"Model comparison results:\n{resultsDF.to_string()}"
+        log.info(strResults)
+        if resultWriter is not None:
+            suffix = "crossval" if useCrossValidation else "simple-eval"
+            strResults += "\n\n" + "\n\n".join([f"{model.getName()} = {str(model)}" for model in models])
+            resultWriter.writeTextFile(f"model-comparison-results-{suffix}", strResults)
+        return resultsDF
+
     def compareModelsCrossValidation(self, models: Sequence[TModel], resultWriter: Optional[ResultWriter] = None) -> pd.DataFrame:
         """
         Compares several models via cross-validation
@@ -212,21 +256,8 @@ class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValData,
         :param resultWriter: a writer with which to store results of the comparison
         :return: a data frame containing evaluation metrics on all models
         """
-        statsList = []
-        for model in models:
-            crossValidationResult = self.performCrossValidation(model, resultWriter=resultWriter)
-            stats = crossValidationResult.getEvalStatsCollection().aggStats()
-            stats["modelName"] = model.getName()
-            statsList.append(stats)
-        resultsDF = pd.DataFrame(statsList).set_index("modelName")
-        strResults = f"Model comparison results:\n{resultsDF.to_string()}"
-        log.info(strResults)
-        if resultWriter is not None:
-            resultWriter.writeTextFile("model-comparison-results", strResults)
-        return resultsDF
+        return self.compareModels(models, resultWriter=resultWriter, useCrossValidation=True)
 
-    # TODO: a parameter predictedVarName seemed to have been present at some point but has now disappeared,
-    #   should we resuscitate it?
     def createPlots(self, data: Union[TEvalData, TCrossValData], showPlots=True, resultWriter: Optional[ResultWriter] = None, subtitlePrefix: str = ""):
         """
         Creates default plots that visualise the results in the given evaluation data
@@ -269,14 +300,14 @@ class EvaluationUtil(ABC, Generic[TModel, TEvaluator, TEvalData, TCrossValData,
         pass
 
 
-class RegressionEvaluationUtil(EvaluationUtil[VectorRegressionModel, VectorRegressionModelEvaluator, VectorRegressionModelEvaluationData, VectorRegressionModelCrossValidationData, RegressionEvalStats]):
+class RegressionEvaluationUtil(EvaluationUtil[VectorRegressionModel, VectorRegressionModelEvaluator, VectorRegressionModelEvaluationData, VectorRegressionModelCrossValidator, VectorRegressionModelCrossValidationData, RegressionEvalStats]):
     def _createEvalStatsPlots(self, evalStats: RegressionEvalStats, resultCollector: EvaluationUtil.ResultCollector, subtitle=None):
         resultCollector.addFigure("error-dist", evalStats.plotErrorDistribution(titleAdd=subtitle))
         resultCollector.addFigure("heatmap-gt-pred", evalStats.plotHeatmapGroundTruthPredictions(titleAdd=subtitle))
         resultCollector.addFigure("scatter-gt-pred", evalStats.plotScatterGroundTruthPredictions(titleAdd=subtitle))
 
 
-class ClassificationEvaluationUtil(EvaluationUtil[VectorClassificationModel, VectorClassificationModelEvaluator, VectorClassificationModelEvaluationData, VectorClassificationModelCrossValidationData, ClassificationEvalStats]):
+class ClassificationEvaluationUtil(EvaluationUtil[VectorClassificationModel, VectorClassificationModelEvaluator, VectorClassificationModelEvaluationData, VectorClassificationModelCrossValidator, VectorClassificationModelCrossValidationData, ClassificationEvalStats]):
     def _createEvalStatsPlots(self, evalStats: ClassificationEvalStats, resultCollector: EvaluationUtil.ResultCollector, subtitle=None):
         resultCollector.addFigure("confusion-matrix", evalStats.plotConfusionMatrix(titleAdd=subtitle))
 
@@ -290,11 +321,10 @@ class MultiDataEvaluationUtil:
         self.inputOutputDataDict = inputOutputDataDict
         self.keyName = keyName
 
-    def compareModelsCrossValidation(self, isRegression, modelFactories: Sequence[Callable[[], VectorModel]],
+    def compareModelsCrossValidation(self, modelFactories: Sequence[Callable[[], VectorModel]],
             resultWriter: Optional[ResultWriter] = None, writePerDatasetResults=True,
             crossValidatorParams: Optional[Dict[str, Any]] = None, columnNameForModelRanking: str = None, rankMax=True) -> Tuple[pd.DataFrame, pd.DataFrame]:
         """
-        :param isRegression: flag indicating whether the models to evaluate are regression models
         :param modelFactories: a sequence of factory functions for the creation of models to evaluate
         :param resultWriter: a writer with which to store results
         :param writePerDatasetResults: whether to use resultWriter (if not None) in order to generate detailed results for each
@@ -308,8 +338,15 @@ class MultiDataEvaluationUtil:
         allResults = pd.DataFrame()
         for key, inputOutputData in self.inputOutputDataDict.items():
             log.info(f"Evaluating models for {key}")
-            ev = createEvaluationUtil(inputOutputData, isRegression=isRegression, crossValidatorParams=crossValidatorParams)
             models = [f() for f in modelFactories]
+            modelsAreRegression = [model.isRegressionModel() for model in models]
+            if all(modelsAreRegression):
+                isRegression = True
+            elif not any(modelsAreRegression):
+                isRegression = False
+            else:
+                raise ValueError("The models have to be either all regression models or all classification, not a mixture")
+            ev = createEvaluationUtil(inputOutputData, isRegression=isRegression, crossValidatorParams=crossValidatorParams)
             childResultWriter = resultWriter.childForSubdirectory(key) if writePerDatasetResults else None
             df = ev.compareModelsCrossValidation(models, resultWriter=childResultWriter)
             df[self.keyName] = key
