@@ -7,11 +7,11 @@ import re
 from .util.string import orRegexGroup
 from .sklearn.sklearn_base import AbstractSkLearnMultipleOneDimVectorRegressionModel, AbstractSkLearnVectorClassificationModel
 
-_log = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
 
 class LightGBMVectorRegressionModel(AbstractSkLearnMultipleOneDimVectorRegressionModel):
-    _log = _log.getChild(__qualname__)
+    log = log.getChild(__qualname__)
 
     def __init__(self, categoricalFeatureNames: Optional[Union[Sequence[str], str]] = None, random_state=42, num_leaves=31, **modelArgs):
         """
@@ -40,12 +40,12 @@ class LightGBMVectorRegressionModel(AbstractSkLearnMultipleOneDimVectorRegressio
             categoricalFeatureNames = [col for col in cols if re.match(self._categoricalFeatureNameRegex, col)]
             colIndices = [cols.index(f) for f in categoricalFeatureNames]
             args = {"cat_column": colIndices}
-            self._log.info(f"Updating model parameters with {args}")
+            self.log.info(f"Updating model parameters with {args}")
             self.modelArgs.update(args)
 
 
 class LightGBMVectorClassificationModel(AbstractSkLearnVectorClassificationModel):
-    _log = _log.getChild(__qualname__)
+    log = log.getChild(__qualname__)
 
     def __init__(self, categoricalFeatureNames: Sequence[str] = None, random_state=42, num_leaves=31, **modelArgs):
         """
@@ -75,5 +75,5 @@ class LightGBMVectorClassificationModel(AbstractSkLearnVectorClassificationModel
             categoricalFeatureNames = [col for col in cols if re.match(self._categoricalFeatureNameRegex, col)]
             colIndices = [cols.index(f) for f in categoricalFeatureNames]
             args = {"cat_column": colIndices}
-            self._log.info(f"Updating model parameters with {args}")
+            self.log.info(f"Updating model parameters with {args}")
             self.modelArgs.update(args)
\ No newline at end of file
